@@ -45,10 +45,10 @@ gammaCKpar <- function(matAp, matBp, n.cores = NULL, cut.a = 0.92, cut.p = 0.88,
     matBp[matBp == ""] <- NA
 
     if(sum(is.na(matAp)) == length(matAp) | length(unique(matAp)) == 1){
-        stop("You have no variation in this variable, or all observations are missing in dataset A.")
+        cat("WARNING: You have no variation in this variable, or all observations are missing in dataset A.")
     }
     if(sum(is.na(matBp)) == length(matBp) | length(unique(matBp)) == 1){
-        stop("You have no variation in this variable, or all observations are missing in dataset B.")
+        cat("WARNING: You have no variation in this variable, or all observations are missing in dataset B.")
     }
     
     if(!(method %in% c("jw", "jaro", "lv"))){
